/***************************************************************************/
/*                                                                         */
/*  File: AITank.cpp                                                       */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
*/
/***************************************************************************/

#include "AITank.h"

#include "Vector3.h"
#include "Matrix4.h"
#include "Game.h"

/***************************************************************************/

bool AITank::Create(Land* land, AI_TANK_TYPE type)
{
	// Call the base class
	bool ret = Create(land);

	m_brainType = type;
	SetRandBrainTime(); 

	SetPos(VECTOR3(0, 0, -5));

	return ret;
}// End Create(..)

/***************************************************************************/


void AITank::Update()
{
	m_brainChangeTime -= g_timeStep;
	if (m_brainChangeTime <= 0)
	{
		m_brainType = (AI_TANK_TYPE)((int)RandRange((int)AI_CIRCLE, (int)AI_ATTACK)+1);
		SetRandBrainTime();
	}

	switch (m_brainType)
	{
		case AI_CIRCLE:
		{
			UpdateBrainCircle();
		}
		break;

		case AI_ROAM:
		{
			UpdateBrainRoam();
		}
		break;

		case AI_ATTACK:
		{
			UpdateAttack();
		}
		break;
	}//End switch(..)

	Tank::Update();

}// End Update(..)

/***************************************************************************/


void AITank::UpdateBrainCircle()
{
	VECTOR3 min, max;
	m_land->GetMinMax(min, max);

	VECTOR3 rad = (max-min)*0.5f;

	VECTOR3 centre = min + rad;
	rad.y = 0.0f;
	

	// We want it less than the max, ideally it would be 0.707, but would
	// touch the closest edge
	float radius = rad.length()*0.6f;

	VECTOR3 refDir = VECTOR3(0,0,1);
	MATRIX4 rot;
	rot.RotateY(m_rotStep);
	TransformCoord(&refDir, &refDir, &rot);



	VECTOR3 target = centre + refDir * radius;
	VECTOR3 dir = target - GetPos();
	dir.y=0;
	float dist = dir.length();
	if (dist < 2.5f)
	{
		m_rotStep += 3.14f / 14.0f;
	}
	dir.normalize();

	// Do some simple collision detection - if whatever we are hitting
	// is actually on top of our target point, then
	// grab next target along
	if (m_hit>=0)
	{
		VECTOR3 dir = g_Tanks[m_hit]->GetPos() - target;
		dir.y = 0.0f;
		float gap = g_Tanks[m_hit]->GetRadius() + m_radius + 0.3f;
		float len = dir.length();
		if (len < gap)
		{
			m_rotStep -= 2*(3.14f / 14.0f);
		}
	}
	

	SetDir(dir);
	SetForce(2.0f);



}//End UpdateBrainCircle(..)

/***************************************************************************/


void AITank::UpdateBrainRoam()
{
	VECTOR3 min, max;
	m_land->GetMinMax(min, max);

	VECTOR3 rad = (max-min)*0.5f;

	VECTOR3 centre = min + rad;
	rad.y = 0.0f;
	

	// We want it less than the max, ideally it would be 0.707, but would
	// touch the closest edge - size of our world, min max, radius
	float radius = rad.length()*0.6f;


	VECTOR3 dir = m_target - GetPos();
	dir.y = 0.0f;
	float dist = dir.length();

	if (dist < 0.3f)
	{
		m_target = VECTOR3(RandRange(min.x,max.x),
						   0.0f,
						   RandRange(min.z,max.z));
	}
	dir.normalize();

	// Do some simple collision detection - if whatever we are hitting
	// is actually on top of our target point, then
	// grab next target along
	if (m_hit>=0)
	{
		VECTOR3 dir = g_Tanks[m_hit]->GetPos() - m_target;
		dir .y = 0.0f;
		float gap = g_Tanks[m_hit]->GetRadius() + m_radius + 0.3f;
		float len = dir.length();
		if (len < gap)
		{
			m_target = VECTOR3(	RandRange(min.x,max.x),
								0.0f,
								RandRange(min.z,max.z));
		}
	}
	
	SetDir(dir);
	SetForce(2.0f);



}//End UpdateBrainRoam(..)

/***************************************************************************/


void AITank::UpdateAttack()
{

	VECTOR3 dirToEnemy = GetPos() - g_TankPlayer.GetPos();
	float distToEnemy = dirToEnemy.length();

	dirToEnemy.normalize();
	VECTOR3 curFacingDir = GetGradualDir();
	curFacingDir.y = 0.0f;
	curFacingDir.normalize();

	

	SetDir(-dirToEnemy);

	if (distToEnemy < 10.0f)
	{
		float ang = acosf( curFacingDir.dot(-dirToEnemy) );
		if ( (abs(ang) < 0.1f)  || (m_hit>=0))
		{
			float fireStrength = RandRange(3.0f, 15.0f);
			Fire( fireStrength );

			m_brainType = (AI_TANK_TYPE)((int)RandRange((int)AI_CIRCLE, (int)AI_ATTACK));
			SetRandBrainTime();
		}
		SetForce(0.5f);
	}
	else
	{
		SetForce(2.0f);
	}

	
	

}// End UpdateAttack(..)