/***************************************************************************/
/*                                                                         */
/*  File: Bullets.h                                                        */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/
/*

*/
/***************************************************************************/
#ifndef BULLETS_H
#define BULLETS_H


#include <windows.h>    // opengl has issues if we don't include it!
#include <GL/gl.h>
//#include <GL/glu.h>	// gluLookAt(..), glPerspective(..)


#include "Vector3.h"
#include "GLDraw.h"		// DrawUnitCube(..)
#include "Explode.h"	// Explode class


#define MAX_BULLETS 5


/***************************************************************************/
class SingleBullet
{
public:
	VECTOR3 m_pos;
	VECTOR3 m_vel;
	bool	m_active;

};// End SingleBullet Class


/***************************************************************************/

extern float g_timeStep;


class Bullets
{
protected:
	SingleBullet m_bulletList[MAX_BULLETS];
	Explode		 m_explosions;

public:
	Bullets()
	{
		for (int i=0; i<MAX_BULLETS; i++)
		{
			m_bulletList[i].m_active=false;
		}
	}

	void AddBullet(const VECTOR3& pos, const VECTOR3& vel)
	{
		for (int i=0; i<MAX_BULLETS; i++)
		{
			if (m_bulletList[i].m_active==false)
			{
				m_bulletList[i].m_pos=pos;
				m_bulletList[i].m_vel=vel;
				m_bulletList[i].m_active=true;
				break;
			}
		}
	}

	void Update()
	{
		m_explosions.Update();
		for (int i=0; i<MAX_BULLETS; i++)
		{
			if (m_bulletList[i].m_active)
			{
				VECTOR3 gravity = VECTOR3(0, -9.8f, 0);

				m_bulletList[i].m_pos += g_timeStep*m_bulletList[i].m_vel;


				static float friction = 0.2f;
				//m_bulletList[i].m_vel = m_bulletList[i].m_vel*0.998f;
				m_bulletList[i].m_vel.x = m_bulletList[i].m_vel.x - friction*g_timeStep;
				m_bulletList[i].m_vel.z = m_bulletList[i].m_vel.z - friction*g_timeStep;

				m_bulletList[i].m_vel += g_timeStep*gravity;

				if (m_bulletList[i].m_pos.y < 0.0f)
				{
					m_bulletList[i].m_active=false;
					m_explosions.AddExplode(m_bulletList[i].m_pos);
				}
			}
		}
	}

	void Render()
	{
		m_explosions.Render();
		for (int i=0; i<MAX_BULLETS; i++)
		{
			if (m_bulletList[i].m_active)
			{
				RenderSingleBullet(m_bulletList[i].m_pos);
			}
		}
	}


	void RenderSingleBullet(VECTOR3& pos)
	{
		glPushMatrix();
		glTranslated(pos.x,pos.y,pos.z);
		float sz = 0.1f;
		glScalef (sz, sz, sz);
		glLineWidth(1.0);
		glColor3d(1.0,0.0,0.0);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		DrawUnitCube();
		glPopMatrix();
	}

};//End Bullets Class


#endif //BULLETS_H