/***************************************************************************/
/*                                                                         */
/*  File: Game.cpp                                                         */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
	Main Game File - This is where we update all our elements in the game
	do any creation of things, such as land, tanks and call render
	on them all or update....heart of the whole game is here.

*/
/***************************************************************************/


#include <math.h>		// cosf(..) sinf(..), sqrt(..)
#include <stdio.h>      // sprintf(..)

#include <windows.h>    // opengl has issues if we don't include it!
//#include <GL/gl.h>
#include <GL/glu.h>		// gluLookAt(..), glPerspective(..)
//#include <GL/glaux.h>	
//#include <GL/glut.h>	// glutWireCube(..)


#include "Vector3.h"	// VECTOR3
#include "Land.h"		// Land class
#include "GLDraw.h"		// Various simple draw functions
#include "Tank.h"		// Tank class
#include "Matrix4.h"	// MATRIX4 class
#include "AITank.h"


// OpenGL librarys
#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")


// Declared in main.cpp - used to display simple debug information to the screen
// such as variable values etc
extern void DebugPrint( char * str, float x, float y);


static float		g_fZoom			= 5.70f;
float				g_timeStep		= 1.0f;		// Timing 

static float		g_roty			= 0.0f;
static float		g_force			= 0.0f;
static float		g_fireAngle		= 3.14f/3.0f;
static bool			g_fire			= false;
static float		g_charge		= 0.0f;
static int			g_playerHits	= 0;		// How many times the player is hit
static int			g_enemyHits		= 0;		// How many times you hit the enemy

bool				g_bWire			= false;	// Wire frame rendering on/off
bool				g_bWire_On_Solid = false;	// Wire frame on top of a solid render
bool				g_bSurface		= true;		// Turn surface rendering on/off
bool				g_bSolidTank	= true;	// Toggle drawing a full model tank or not

static Land			g_Land;

Tank				g_TankPlayer;
AITank				g_TankBot0;
AITank				g_TankBot1;


// Keep a single list of our tanks, so we can loop around them
Tank*		g_Tanks[3] = 
{
	&g_TankPlayer,
	&g_TankBot0,
	&g_TankBot1
};


/***************************************************************************/

bool Create()
{
	g_Land.Create(	"media\\land\\earthheightmap.bmp", 
					"media\\land\\earthcolormap.bmp", 
					2.0, 2.0, 9.0);

	g_Land.Move(-16.0f, 0.0f, -13.0f);

	g_TankPlayer.Create(&g_Land, "player0");
	g_TankBot0.Create(&g_Land, AITank::AI_CIRCLE);
	g_TankBot1.Create(&g_Land, AITank::AI_ROAM);

	//CreateFairway();

	return true;
}// End Create

/***************************************************************************/

bool Release()
{
	g_Land.Release();

	for (int i=0; i<sizeof(g_Tanks)/sizeof(Tank*); i++)
	{
		g_Tanks[i]->Release();
	}

	return true;
}// End Release()



/***************************************************************************/
/*                                                                         */
/*  Name: KeyBoard()                                                       */
/*                                                                         */
/*  Used to accept keyboard input and alter some global variables - used   */
/*  for simple analysis of our code - turning wire frame rendering on/off  */
/*  adjusting variables which are used in the demo.                        */
/*                                                                         */
/***************************************************************************/
void KeyBoard()
{
	// Controlling the rendering
	if( GetKeyState('W')		& 0x80 ) g_bWire=!g_bWire;
	if( GetKeyState('Q')		& 0x80 ) g_bWire_On_Solid=!g_bWire_On_Solid;
	if( GetKeyState('O')		& 0x80 ) g_bSurface=!g_bSurface;

	// Controlling movement
	float speed = 2.8f * g_timeStep;
	g_force = 0.0f;
	if( GetKeyState(VK_RIGHT)	& 0x80 )  g_roty-=speed;
	if( GetKeyState(VK_LEFT)	& 0x80 )  g_roty+=speed;

	if( GetKeyState(VK_UP)		& 0x80 )  g_force = 5.0f; //g_vel=min(g_vel+speed,  1.3f);
	if( GetKeyState(VK_DOWN)	& 0x80 )  g_force = -5.0f; //g_vel=max(g_vel-speed, -1.3f);

	g_fire = true;
	if( GetKeyState(VK_SPACE)	& 0x80 )
	{ 
		g_charge= min(g_charge+speed*3.0f, 15.0f); 
		g_fire = false;
	}

	if( GetKeyState('Z')		& 0x80 ) g_fZoom=min(g_fZoom+speed*6.0f, 50.0f);
	if( GetKeyState('X')		& 0x80 ) g_fZoom=max(g_fZoom-speed*6.0f, 3.81f);

	if( GetKeyState('P')		& 0x80 ) g_fireAngle=min(g_fireAngle+speed, 3.14f/2.4f);
	if( GetKeyState('L')		& 0x80 ) g_fireAngle=max(g_fireAngle-speed, 0.6f);

	if( GetKeyState('T')		& 0x80 ) g_bSolidTank=!g_bSolidTank;
}//End KeyBoard()


VECTOR3 g_tempPos = VECTOR3(0,0,0);

/***************************************************************************/
/*                                                                         */
/*  Name: UpdateCamera(..)                                                 */
/*  Date: 07-02-06                                                         */
/*                                                                         */
/***************************************************************************/
void UpdateCamera()
{	// Setting World/Camera Constants	

	// Current viewport:x, y, width, height
	glViewport(0, 0, 640, 480);    
 
	// Next we set up for a perspective view
	glMatrixMode(GL_PROJECTION);         // Select the Projection Matrix
	glLoadIdentity();                    // Reset(init) the Projection Matrix

	gluPerspective(45.0f, 640/480, 0.1f, 100.0f);       
	glMatrixMode(GL_MODELVIEW);          // Select the modelview matrix
	glLoadIdentity();                    // Reset (init) the modelview matrix

	VECTOR3 pos = g_TankPlayer.GetPos();
	VECTOR3 dir = g_TankPlayer.GetDir();

	static float ypos = pos.y;
	ypos = ypos + g_timeStep*0.5f*(pos.y-ypos);

	VECTOR3 at = VECTOR3(pos.x,   ypos+4.0f,   pos.z);
	VECTOR3 eye= VECTOR3(pos.x,  g_fZoom, pos.z);

	static VECTOR3 prevdir = dir;
	prevdir = prevdir + g_timeStep*(dir - prevdir);


	// Debugging, force camera to always face the forward
	// z direction
	//prevdir = VECTOR3(0,0,1);

	eye -= prevdir*10.0f;
	eye.y += g_fZoom;

	gluLookAt(	eye.x, eye.y, eye.z,	  // eye - where our cam is positioned
				at.x, at.y, at.z,		  // Centre of cam, what its looking at
				0.0f, 1.0f, 0.0f);		  // Up Vector

	
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glShadeModel(GL_SMOOTH);             // Enable smooth shading
	glClearDepth(1.0f);                  // Clear depth buffer
	glEnable(GL_DEPTH_TEST);             // Enables depth test
	glDepthFunc(GL_LEQUAL);              // Type of depth test to perform


	// OpenGL Translation and Roation API's
	glTranslated(0, 0, 0);
	glRotatef(0, 1.0f, 0.0f, 0.0f);
	glRotatef(0, 0.0f, 1.0f, 0.0f);
	glRotatef(0, 0.0f, 0.0f, 1.0f);

	glShadeModel(GL_SMOOTH);

	GLfloat LightAmbient[]=  { 0.9f, 0.9f, 0.9f, 1.0f };	// Ambient Light Values, the values define the amount of P.C light
	GLfloat LightDiffuse[]=	 { 0.8f, 0.8f, 0.8f, 1.0f };	// Diffuse Light Values
	GLfloat LightPosition[]= { 0.0f, 50.0f, 0.0f, 1.0f };	// Light Position, x, y, z and last leave at 1.0 (code junk)

	LightPosition[0] = g_TankPlayer.GetPos().x;
	LightPosition[1] = g_TankPlayer.GetPos().y + 2;
	LightPosition[2] = g_TankPlayer.GetPos().z;

	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);	
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);		
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);	
	
	glLightModelfv (GL_LIGHT_MODEL_AMBIENT, LightAmbient);

	glEnable(GL_LIGHT1);  
	glEnable(GL_NORMALIZE);
	glEnable(GL_LIGHTING);

}// End UpdateCameara(..)


DrawString* g_str = NULL;

/***************************************************************************/
/*                                                                         */
/* Update()                                                                */
/* Our main render loop, which gets called over and over agian to do our   */
/* drawing...clears the screen, draws the data, then presents it..         */
/*                                                                         */
/***************************************************************************/
void Update()
{
	/*
	float nowTime = (float)GetTickCount();
	static float oldTime = nowTime + 1.0f;
	g_timeStep = nowTime - oldTime;
	oldTime = nowTime;

	// Convert from millisec to sec
	g_timeStep /= 1000.0f;
	*/

	// We use a more accurate timing information, QueryPerformanceCounter
	// instead of GetTickCount() so we get smoother animations
	LARGE_INTEGER freq;
	QueryPerformanceFrequency(&freq);

	LARGE_INTEGER nTime;
	QueryPerformanceCounter(&nTime);

	static LARGE_INTEGER oTime = nTime;

	double delay = (double(nTime.QuadPart - oTime.QuadPart)/freq.QuadPart);
	oTime=nTime;


	g_timeStep = (float)delay;



	if (g_timeStep > 1.0f)
	{
		g_timeStep = 1.0f;
	}

	UpdateCamera();											// Update view/world mat
	
	KeyBoard();												// Get keyboard input

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);					// Set background to black
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear screen

	//DrawFairway();
										
	g_Land.Render();										// Render Land & Shapes

	for (int i=0; i<sizeof(g_Tanks)/sizeof(Tank*); i++)		// Draw a test tank
	{
		g_Tanks[i]->Update();
		g_Tanks[i]->Render();										
	}
	


	VECTOR3 refDir = VECTOR3(0,0,1);
	MATRIX4 rot;
	rot.RotateY(g_roty);
	TransformCoord(&refDir, &refDir, &rot);

	g_TankPlayer.SetDir(refDir);
	g_TankPlayer.SetForce(g_force);
	g_TankPlayer.SetGunUpAngle(g_fireAngle);


	if ((g_charge> 0.1f) && (g_fire==true))
	{
		g_TankPlayer.Fire(g_charge);
		g_charge=0.0f;
	}
	

	// Display information to the user - show our user some feedback of 
	// what is happening, fps, land height, enemies in vacinity etc.
	static char buf[300];
	static DrawString str;
	g_str = &str;

	sprintf(buf, "Debug Mode...");
	g_str->Print(buf, 5, 10);

	sprintf(buf, "Cursor Keys <> move...");
	g_str->Print(buf, 5, 20);
	sprintf(buf, "Camera Zoom Z&X...");
	g_str->Print(buf, 5, 30);
	sprintf(buf, "TimeStep.%.2f.. (%.2f)", g_timeStep, delay);
	g_str->Print(buf, 5, 40);
	sprintf(buf, "Fire Charge: %.2f (Hold SpaceBar)", g_charge);
	g_str->Print(buf, 5, 50);
	sprintf(buf, "Fire Angle: %.1f L&P", g_fireAngle);
	g_str->Print(buf, 5, 60);

	static float updateFPS = 0.0f;
	updateFPS += (float)delay;
	static float fps = 1.0f/(float)delay;
	if (updateFPS>1.0f)
	{
		updateFPS=0.0f;
		fps = 1.0f/(float)delay;
	}


	extern HWND g_hwnd; // top of main.cpp
	static char title[256] = {0};
	sprintf(title, "Game Demo Tanks - FPS:%.1f,  TimeStep:%.2f", fps, g_timeStep);
	SetWindowText(g_hwnd, title);
	



}//End Update()
