/***************************************************************************/
/*                                                                         */
/*  File: Land.cpp                                                         */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/

#include <windows.h>	// BITMAPFILEHEADER structure define
#include <stdio.h>		// fopen(..), fread(..)
#include <float.h>		// _isnan(..)

#include "Land.h"

#include <GL/gl.h>
#include <GL/glu.h>		// gluLookAt(..), glPerspective(..)


#include <gl\glaux.h>		// Header File For The Glaux Library

// OpenGL librarys
#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")


extern bool g_bWire_On_Solid;
extern bool g_bWire;


AUX_RGBImageRec *LoadBMP(char *Filename)					// Loads A Bitmap Image
{
	FILE *File=NULL;							// File Handle

	if (!Filename)								// Make Sure A Filename Was Given
	{
		return NULL;							// If Not Return NULL
	}

	File=fopen(Filename,"r");						// Check To See If The File Exists

	if (File)								// Does The File Exist?
	{
		fclose(File);							// Close The Handle
		return auxDIBImageLoad(Filename);				// Load The Bitmap And Return A Pointer
	}
	return NULL;								// If Load Failed Return NULL
}




struct stBITMAP_FILE
{
	BITMAPFILEHEADER bitmapfileheader;  // this contains the bitmapfile header
	BITMAPINFOHEADER bitmapinfoheader;  // this is all the info including the palette
	PALETTEENTRY     palette[256];      // we will store the palette here
	UCHAR            *buffer;           // this is a pointer to the data
};


static bool Load_Bitmap_File(stBITMAP_FILE* pBitmap, char *szFileName)
{
	memset(pBitmap, 0, sizeof(stBITMAP_FILE));

	FILE* fp = fopen(szFileName, "rb");

	fread(&pBitmap->bitmapfileheader, 1, sizeof(BITMAPFILEHEADER), fp);
	fread(&pBitmap->bitmapinfoheader, 1, sizeof(BITMAPINFOHEADER), fp);


	// now load the color palette if there is one
	if (pBitmap->bitmapinfoheader.biBitCount == 8)
	{
		int iMaxColours = 256;
		fread(&pBitmap->palette, 1, iMaxColours*sizeof(PALETTEENTRY), fp );

		// now set all the flags in the palette correctly and fix the reversed 
		// BGR RGBQUAD data format
		for (int index=0; index < iMaxColours; index++)
		{
			// reverse the red and green fields
			int temp_color                 = pBitmap->palette[index].peRed;
			pBitmap->palette[index].peRed  = pBitmap->palette[index].peBlue;
			pBitmap->palette[index].peBlue = temp_color;
		       
			// always set the flags word to this
			pBitmap->palette[index].peFlags = PC_NOCOLLAPSE;
		} // end for index

	} // end if

	// finally the image data itself
	fseek(fp, -(int)(pBitmap->bitmapinfoheader.biSizeImage), SEEK_END );

	// now read in the image
	if (pBitmap->bitmapinfoheader.biBitCount==8 || pBitmap->bitmapinfoheader.biBitCount==16) 
	{
		// allocate the memory for the image
		pBitmap->buffer = new unsigned char[pBitmap->bitmapinfoheader.biSizeImage];

		// now read it in
		fread( pBitmap->buffer, 1,  pBitmap->bitmapinfoheader.biSizeImage,  fp);

	} // end if


	fclose(fp);

	// return success
	return true;

} // end Load_Bitmap_File



/***************************************************************************/
/*                                                                         */
/*  Class Definition                                                       */
/*                                                                         */
/***************************************************************************/


bool Land::Create(	char* szHeightMapFile,
					char* szTextureMapFile,
					float fStepWidth,
					float fStepLength,
					float fHeightScale)
{
	//----------------------------------------//
	//------Save passed screen object---------//

	m_fStepWidth   = fStepWidth;
	m_fStepLength  = fStepLength;
	m_fHeightScale = fHeightScale;

	m_vPos = VECTOR3(0,0,0);

	//--------Load our Texture----------------//

	AUX_RGBImageRec *TextureImage[1];
	if (TextureImage[0]=LoadBMP(szTextureMapFile))
	{
		glGenTextures(1, &m_texture);	

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, m_texture);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST); 


		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data); 
	}

	//---------Get Bitmap Details-------------//
	HBITMAP b = ( HBITMAP ) LoadImage (NULL , szHeightMapFile, IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE|LR_LOADFROMFILE ) ;

	if( b==NULL)
	{
		MessageBox(0, "Error Loading Bitmap", "Error Loading Bitmap", 0);
		return false;
	}

	BITMAP bitmap ;
	GetObject ( b,  sizeof ( BITMAP ),  &bitmap ) ;
	m_Columns = bitmap.bmWidth ;
	m_Rows    = bitmap.bmHeight ;

	//---------Mesh Details-------------//

	m_iNumVertices = m_Columns*m_Rows;
	m_iNumPolys = ((m_Columns-1)*(m_Rows-1)*2);

	//---------Create Index Buffer-------//
	m_pIndexBuffer = new WORD[3*m_iNumPolys];

	memset(m_pIndexBuffer, 0, 3*m_iNumPolys*sizeof(WORD));


	//---------Create Vertex Buffer------//

	m_pVertexBuffer = new stVertex[m_iNumVertices];
	memset(m_pVertexBuffer, 0, m_iNumVertices*sizeof(stVertex));


	//----Genereate the vertex list------//
	stVertex *pVertexList = m_pVertexBuffer;

	stBITMAP_FILE HeightMap;
	Load_Bitmap_File(&HeightMap, szHeightMapFile);
		

	volatile int indx=0;
	for( int iCurrRow=0; iCurrRow< m_Rows; iCurrRow++ )
	{
		for( int iCurrCol=0; iCurrCol< m_Columns; iCurrCol++ )
		{
			indx = (iCurrRow* (m_Columns)) + iCurrCol;

			pVertexList[indx].x = (float)iCurrCol * fStepWidth;
			pVertexList[indx].z = (float)iCurrRow * fStepLength;
				
			float fColourValue = (float)HeightMap.buffer[iCurrCol + (iCurrRow*m_Columns)]/255.0f;
			pVertexList[indx].y = fColourValue * fHeightScale;

			int iIndex = HeightMap.buffer[indx];

			UCHAR r = HeightMap.palette[iIndex].peRed;
			UCHAR g = HeightMap.palette[iIndex].peGreen;
			UCHAR b = HeightMap.palette[iIndex].peBlue;

			pVertexList[indx].r = ((float)r)/0xff;
			pVertexList[indx].g = ((float)g)/0xff;
			pVertexList[indx].b = ((float)b)/0xff;

			// If we have a texture map texture coordinates:
			{
				// I manually loaded in the texture...and the y is inverted,
				// as I didn't invert it in the loader...I've done it here :)
				/*
				pVertexList[indx].tu = (float)iCurrCol/(float)m_Columns;
				pVertexList[indx].tv = 1 - (float)iCurrRow/(float)m_Rows;
				*/

				float tu = (float)iCurrCol/(float)m_Columns;
				float tv = (float)iCurrRow/(float)m_Rows;

				pVertexList[indx].tu = tu;
				pVertexList[indx].tv = tv;

			}//end if

		}// end inner for loop
	}// end outer for loop
		
	delete[] HeightMap.buffer;

	//------Genererate Polygon List-------------//
		
	WORD *pPolyList = m_pIndexBuffer;

	indx = 0;
		
	for( int iCurrRow=0; iCurrRow< m_Rows-1; iCurrRow++ )
	{
		for( int iCurrCol=0; iCurrCol< m_Columns-1; iCurrCol++ )
		{
			int iBasePolyIndex = ( iCurrRow* (m_Columns)) + iCurrCol;

			pPolyList[indx] = iBasePolyIndex;
			indx++;
			pPolyList[indx] = iBasePolyIndex+m_Columns;
			indx++;
			pPolyList[indx] = iBasePolyIndex+m_Columns+1;
			indx++;

			pPolyList[indx] = iBasePolyIndex;
			indx++;
			pPolyList[indx] = iBasePolyIndex+m_Columns+1;
			indx++;
			pPolyList[indx] = iBasePolyIndex+1;
			indx++;
		}
	}

	DeleteObject(&bitmap);


	
	//---------Compute the Max and Mins---------//

	m_max = VECTOR3(-9999.0f, -9999.0f, -9999.0f);
	m_min = VECTOR3( 9999.0f,  9999.0f,  9999.0f);

	// Lets work out the min and max for our 3d mesh world
	for (int i=0; i<m_iNumPolys*3; i+=3)
	{
		WORD iA = m_pIndexBuffer[i+0];
		WORD iB = m_pIndexBuffer[i+1];
		WORD iC = m_pIndexBuffer[i+2];

		stVertex* tA = &m_pVertexBuffer[iA];
		stVertex* tB = &m_pVertexBuffer[iB];
		stVertex* tC = &m_pVertexBuffer[iC];

		m_min.x = min(m_min.x, min(tA->x,min(tB->x,tC->x)));
		m_min.y = min(m_min.y, min(tA->y,min(tB->y,tC->y)));
		m_min.z = min(m_min.z, min(tA->z,min(tB->z,tC->z)));
		
		m_max.x = max(m_max.x, max(tA->x,max(tB->x,tC->x)));
		m_max.y = max(m_max.y, max(tA->y,max(tB->y,tC->y)));
		m_max.z = max(m_max.z, max(tA->z,max(tB->z,tC->z)));
	}

	return true;
} // End of Create(..)




void Land::Release()
{
	if (m_pIndexBuffer)
	{
		delete[] m_pIndexBuffer;
		m_pIndexBuffer=NULL;
	}

	if (m_pVertexBuffer)
	{
		delete[] m_pVertexBuffer;
		m_pVertexBuffer=NULL;
	}

} // End of Release();



void Land::Render()
{
	glLineWidth(1.0);
	glPushMatrix();
	glTranslated(m_vPos.x, m_vPos.y, m_vPos.z);
	glRotatef(0, 1.0f, 0.0f, 0.0f);
	glRotatef(0, 0.0f, 1.0f, 0.0f);
	glRotatef(0, 0.0f, 0.0f, 1.0f);

	// If we have solid and wireframe rendering, or just solid
	if( g_bWire && (!g_bWire_On_Solid) )
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	RenderLand();

	if( g_bWire_On_Solid )
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		RenderLand(0.0f);
	}

	glPopMatrix(); 
}// End Render(..)


// The *c was so I could scale the colour values
void Land::RenderLand(float c)
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_texture);

	glBegin(GL_TRIANGLES);
	for (int i=0; i<m_iNumPolys*3; i+=3)
	{
		WORD iA = m_pIndexBuffer[i+0];
		WORD iB = m_pIndexBuffer[i+1];
		WORD iC = m_pIndexBuffer[i+2];

		stVertex* tA = &m_pVertexBuffer[iA];
		stVertex* tB = &m_pVertexBuffer[iB];
		stVertex* tC = &m_pVertexBuffer[iC];

		glColor3f(tA->r*c, tA->g*c, tA->b*c);
		glTexCoord2f(tA->tu, tA->tv);
		//glNormal3f(nx, ny, nz);
		glVertex3f(tA->x, tA->y, tA->z); 

		glColor3f(tB->r*c, tB->g*c, tB->b*c);
		glTexCoord2f(tB->tu, tB->tv);
		//glNormal3f(nx, ny, nz);
		glVertex3f(tB->x, tB->y, tB->z); 

		glColor3f(tC->r*c, tC->g*c, tC->b*c);
		glTexCoord2f(tC->tu, tC->tv);
		//glNormal3f(nx, ny, nz);
		glVertex3f(tC->x, tC->y, tC->z); 
	}
	glEnd();

}// End of RenderLand()



void Land::Move(float x, float y, float z)
{
	m_vPos.x = x;
	m_vPos.z = z;
}// End of Move(..)



float Land::GetHeightApprox(float x, float z) const
{
	//[1] Originally 0,0 is the bottom left of the grid
	//[2] But we scaled x,z by fStepWidth and fStepLength

	x = x - m_vPos.x;
	z = z - m_vPos.z;

	x = x/m_fStepWidth;
	z = z/m_fStepLength;

	x=x-0.5f;
	z=z-0.5f;

	x = ceilf(x);
	z = ceilf(z);

	if( (z >= m_Columns) || (z<= 0 ) )
		return 0.0f;

	if( (x >= m_Rows) || (x<= 0 ) )
		return 0.0f;

	int v0 = (int)(x + z*m_Columns);
	int v1 = (int)(v0 + 1);

	int v2 = (int)(v0 + m_Columns);
	int v3 = (int)(v1 + m_Columns);

	// Alternative to the average method, which is to take the heighest
	// value instead of the average.
	//int vHeighest = v0;
	//if( v1 > vHeighest ) vHeighest = v1;
	//if( v2 > vHeighest ) vHeighest = v2;
	//if( v3 > vHeighest ) vHeighest = v3;
	//float fAveHeight = pVertexList[vHeighest].y;

	stVertex *pVertexList = m_pVertexBuffer;

	float fAveHeight = 0.25f * (pVertexList[v0].y +
								pVertexList[v1].y +
								pVertexList[v2].y +
								pVertexList[v3].y );

	// Debug to display where on the grid our point is being taken from
	/*
	pVertexList[v0].colour = 0xffff0000;
	pVertexList[v1].colour = 0xffff0000;
	pVertexList[v2].colour = 0xffff0000;
	pVertexList[v3].colour = 0xffff0000;
	*/

	return fAveHeight;
}// End of GetHeightApprox(..)


static __forceinline bool TriangleToLineCollision( VECTOR3 v1, VECTOR3 v2, VECTOR3 v3,
                                                 VECTOR3 p1, VECTOR3 p2,
											     VECTOR3& vCollisionPoint )
{
      // v1-v3 are our points representing our triangle
      // p1-p2 are the two points for our line

      //<Part-1->
      VECTOR3 vA = v2 - v1;
      VECTOR3 vB = v3 - v1;

	  vA.normalize();
	  vB.normalize();

      VECTOR3 vNormal;
	  vNormal = vA.cross(vB);
	 
      float A = vNormal.x;
      float B = vNormal.y;
      float C = vNormal.z;

      float x = v1.x; // Just choose any point from our triangle, as long as 
      float y = v1.y; // its on our plane.
      float z = v1.z;

      // Now we use the magic plane law/formula - e.g. Ax + By + Cz + D = 0;
      // Where ABC - Plane Normals
      //       xyz - point on the plane
      //         D - distance from origin

      float fDistanceFromOrigin = (-A*x - B*y - C*z );

      // Now we use our two points for our line, and make sure that
      // they cross our triangles plane.

      float distance1 = A*p1.x + B*p1.y + C*p1.z + fDistanceFromOrigin;
      float distance2 = A*p2.x + B*p2.y + C*p2.z + fDistanceFromOrigin;

      if( distance1 * distance2 >= 0 ) // As if one was on one side of the plane and the other
            return false;                // was on the other side, then one would have a -ve
                                       // value and the other a +ve, leading to a negative
                                       // result when multiplied together!

      //<Part-2->
      VECTOR3 vLineDirection = p1 - p2;
	  vLineDirection.normalize();


	  
      float fNumerator   = -(A*p1.x + B*p1.y + C*p1.z + fDistanceFromOrigin);
	  float fDenominator = vNormal.dot(vLineDirection);

      // Be careful of divide by zero's! Can be very bad for the computer!
      if( fDenominator == 0.0f )
            return false;

      float fDist = fNumerator / fDenominator;

      VECTOR3 vPoint;
      vPoint.x = p1.x + (vLineDirection.x * fDist);
      vPoint.y = p1.y + (vLineDirection.y * fDist);
      vPoint.z = p1.z + (vLineDirection.z * fDist);

	  vCollisionPoint = vPoint;
      // Excellent thing to know, vPoint is the point in the middle of the triangle
      // where our line clides with the triangle face - but only if there's a collision
      // as its the point where it hits the plane.  The next part determines if its
      // inside the triangles.

      double Angle = 0.0f;

      VECTOR3 vTriangle[3] = { v1, v2, v3 };

      for( int i=0; i<3; i++ )
      {
            VECTOR3 vS = vTriangle[i]       - vPoint;
            VECTOR3 vT = vTriangle[(i+1)%3] - vPoint;

			float d = vS.dot(vT);

			
            //float mag = D3DXVec3Length(&vS) * D3DXVec3Length(&vT);
			float mag = vS.length() * vT.length();


            double tempAngle = acos( d / mag );

            if( _isnan(tempAngle) )
                  tempAngle = 0;

            Angle += tempAngle;
      }

	  float PI=3.1415926535897932f;
	  const double MATCH_FACTOR = 0.9998;

      if( Angle >= (MATCH_FACTOR*2*PI) )
            return true;

      return false;
}// End of TriangleToLineCollision(..)


/*
// BRUTE FORCE METHOD - Works for testing, but is a killer on performance
float Land::GetHeight(float xx, float zz) const
{
	float fHeight=0.0f;

	VECTOR3 vPoint = VECTOR3( xx-m_vPos.x, 0.0f, zz-m_vPos.z);

	stVertex *pVertexList = m_pVertexBuffer;

	VECTOR3 p1 = VECTOR3( vPoint.x,  400, vPoint.z );
	VECTOR3 p2 = VECTOR3( vPoint.x, -400, vPoint.z );

	WORD* pIndex = m_pIndexBuffer;

	VECTOR3 vCollisionPoint;

	int indx = 0;
	for( int i=0; i<m_iNumPolys; i++)
	{
		int v0 = pIndex[indx++];
		int v1 = pIndex[indx++];
		int v2 = pIndex[indx++];

		VECTOR3 vA ( pVertexList[v0].x, pVertexList[v0].y, pVertexList[v0].z );
		vA *= 0.999f;
		VECTOR3 vB ( pVertexList[v1].x, pVertexList[v1].y, pVertexList[v1].z );
		VECTOR3 vC ( pVertexList[v2].x, pVertexList[v2].y, pVertexList[v2].z );

		if( TriangleToLineCollision( vA, vB, vC,
									 p1, p2,
									 vCollisionPoint ) )
		{
			fHeight =  vCollisionPoint.y;
			break;
		}// End of if
	}// End for loop i


	//fHeight = GetHeightApprox(xx, zz);

	return fHeight;
}// End of GetHeight(..)
*/



void Land::GetTri(stTri* t, int iA, int iB, int iC) const
{
	t->v[0] = VECTOR3(m_pVertexBuffer[iA].x, m_pVertexBuffer[iA].y, m_pVertexBuffer[iA].z);
	t->v[1] = VECTOR3(m_pVertexBuffer[iB].x, m_pVertexBuffer[iB].y, m_pVertexBuffer[iB].z);
	t->v[2] = VECTOR3(m_pVertexBuffer[iC].x, m_pVertexBuffer[iC].y, m_pVertexBuffer[iC].z);
}// End GetTri(..)

float Land::GetHeight(float x, float z) const
{
	float fHeight=0.0f;

	float xx = x;
	float zz = z;

	//[1] Originally 0,0 is the bottom left of the grid
	//[2] But we scaled x,z by fStepWidth and fStepLength

	xx = xx - m_vPos.x;
	zz = zz - m_vPos.z;

	xx = xx/m_fStepWidth;
	zz = zz/m_fStepLength;

	xx=xx-0.5f;
	zz=zz-0.5f;

	xx = ceilf(xx);
	zz = ceilf(zz);

	if( (zz >= m_Columns) || (zz<= 0 ) )
		return 0.0f;

	if( (xx >= m_Rows) || (xx<= 0 ) )
		return 0.0f;


	// Now we can work out the index centre of the point on our 3d world
	// mesh, but there are 8 points around it, so we will build up 8
	// triangles around this centre point, which we'll use for collision
	// detection - much better than the brute force method of checking
	// them all!
	stTri tris[8];
	memset(tris, 0, sizeof(tris));

	int i0 = (int)(xx + zz*m_Columns);
	int i1 = (int)(i0 + m_Columns);
	int i2 = (int)(i1 + 1);
	int i3 = (int)(i0 + 1);

	int i4 = (int)(i3 - m_Columns);
	int i5 = (int)(i0 - m_Columns);
	int i6 = (int)(i5 - 1);
	int i7 = (int)(i0 - 1);
	int i8 = (int)(i1 - 1);


	GetTri(&tris[0], i0, i1, i2);
	GetTri(&tris[1], i0, i2, i3);

	GetTri(&tris[2], i5, i0, i3);
	GetTri(&tris[3], i5, i3, i4);

	GetTri(&tris[4], i6, i7, i0);
	GetTri(&tris[5], i6, i0, i5);

	GetTri(&tris[6], i7, i8, i1);
	GetTri(&tris[7], i7, i1, i0);


	// ....
#if 0 // Debug Draw Land Height Check Code
	for (int i=0; i<8; i++)
	{
		glLineWidth(2.0);
		glPushMatrix();
		glTranslated(m_vPos.x, m_vPos.y, m_vPos.z);
		glRotatef(0, 1.0f, 0.0f, 0.0f);
		glRotatef(0, 0.0f, 1.0f, 0.0f);
		glRotatef(0, 0.0f, 0.0f, 1.0f);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

		VECTOR3* tA = &tris[i].v[0];
		VECTOR3* tB = &tris[i].v[1];
		VECTOR3* tC = &tris[i].v[2];

		float h = 0.1f;
		glBegin(GL_TRIANGLES);
		{
			glColor3f(0xff, 0xff, 0);
			glVertex3f(tA->x, tA->y+h, tA->z); 

			glColor3f(0xff, 0, 0);
			glVertex3f(tB->x, tB->y+h, tB->z); 

			glColor3f(0xff, 0, 0);
			glVertex3f(tC->x, tC->y+h, tC->z); 
		}
		glEnd();
		glPopMatrix();
	}

	glLineWidth(3.0);
	glPushMatrix();
	glTranslated(0, 0, 0);
	glRotatef(0, 1.0f, 0.0f, 0.0f);
	glRotatef(0, 0.0f, 1.0f, 0.0f);
	glRotatef(0, 0.0f, 0.0f, 1.0f);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glBegin(GL_LINES);
	{
		glColor3f(0xff, 0, 0xff);
		glVertex3f(x, +5.0f, z); 

		glColor3f(0xff, 0, 0xff);
		glVertex3f(x, -5.0f, z); 
	}
	glEnd();
	glPopMatrix();
#endif

	VECTOR3 vPoint = VECTOR3( x-m_vPos.x, 0.0f, z-m_vPos.z);

	VECTOR3 p1 = VECTOR3( vPoint.x,  400, vPoint.z );
	VECTOR3 p2 = VECTOR3( vPoint.x, -400, vPoint.z );


	VECTOR3 vCollisionPoint;

	for (int i=0; i<8; i++)
	{
		VECTOR3* tA = &tris[i].v[0];
		VECTOR3* tB = &tris[i].v[1];
		VECTOR3* tC = &tris[i].v[2];

		VECTOR3 ttA = *tA;
		ttA = ttA * 0.998f;

		bool hit = false;
		hit = TriangleToLineCollision( ttA, *tB, *tC,
									 p1, p2,
									 vCollisionPoint );
		if (hit)
		{
#if 0
			// Debug draw the current triangle which is being hit
			glLineWidth(2.0);
			glPushMatrix();
			glTranslated(m_vPos.x, m_vPos.y, m_vPos.z);
			glRotatef(0, 1.0f, 0.0f, 0.0f);
			glRotatef(0, 0.0f, 1.0f, 0.0f);
			glRotatef(0, 0.0f, 0.0f, 1.0f);
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

			float h = 0.3f;
			glBegin(GL_TRIANGLES);
			{
				glColor3f(0xff, 0, 0xAA);
				glVertex3f(tA->x, tA->y+h, tA->z); 

				glColor3f(0xff, 0, 0xAA);
				glVertex3f(tB->x, tB->y+h, tB->z); 

				glColor3f(0xff, 0, 0xAA);
				glVertex3f(tC->x, tC->y+h, tC->z); 
			}
			glEnd();
			glPopMatrix();
#endif

			fHeight =  vCollisionPoint.y;
			return fHeight;
		}

	}


	return fHeight;
}// End of GetHeight(..)