/***************************************************************************/
/*                                                                         */
/*  File: Main.cpp                                                         */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/

#define STRICT					// Strict compiling
#define WIN32_LEAN_AND_MEAN		// Tells compiler to produce tigher/smaller code 

#include <windows.h>            
#include <assert.h>
#include <math.h>				// abs(..)


//#include <GL/gl.h>
#include <GL/glu.h>             // OpenGL Header File
//#include <GL/glaux.h>

#pragma comment(lib, "opengl32.lib") // OpenGL librarys
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")

#include "Game.h"

/*****************************************************************************/
/*
   GLOBALS
   As much as we all hate them, its sometimes a little easier...so we've got
   a few globals for the essential of our code.
/*
/*****************************************************************************/

HDC       g_hdc;                  // global device context
HGLRC     g_hrc;                  // global rendering context

HWND      g_hwnd = NULL;          // handle of our window


/***************************************************************************/
// PROTOTYPES                                                              //
/***************************************************************************/

void CreateGl(void);     // - called at the start of our program
void UpdateGl(void){};   // - our mainloop..called over and over again
void ReleaseGl(void);    // - last function we call before we end!


/***************************************************************************/
/*                                                                         */
/* Handle all messages for the main window here                            */
/*                                                                         */
/***************************************************************************/
long _stdcall MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch( uMsg )
    {
        case WM_KEYDOWN:
        {
            switch( wParam )
            {
                case VK_ESCAPE:         // Exit on pressing Escape Key
                    PostQuitMessage(0);
                    break;
			}
        }
		break;

		case WM_DESTROY:
		{
			 // We destroy any game memory we allocated etc, 
			Release();
			ReleaseGl();

            PostQuitMessage(0);
			return 0;
		}
        break;

	}//End switch(..)

    return (long)DefWindowProc(hWnd, uMsg, wParam, lParam);

}//End MsgProc(..)


/***************************************************************************/
/*                                                                         */
/* Program entry point.                                                    */
/*                                                                         */
/***************************************************************************/
int _stdcall WinMain(HINSTANCE i, HINSTANCE, char* k, int) 
{
    MSG msg;
    char szname[] = "Game Demo - Tanks";
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      szname, NULL };
    RegisterClassEx( &wc );
    g_hwnd = CreateWindowEx(WS_EX_APPWINDOW,
                              szname, szname, 
                              WS_OVERLAPPEDWINDOW,//for fullscreen make into WS_POPUP
                              50, 50, 500,500,    //for full screen GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
                              GetDesktopWindow(), NULL, wc.hInstance, NULL);
    
    // Initilise game code
	CreateGl();
	Create();

    ShowWindow(g_hwnd, SW_SHOW);
    UpdateWindow(g_hwnd);     
 
    // Message loop. Note that this has been modified to allow
    // us to execute if no messages are being processed.
    while(1)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (!GetMessage(&msg, NULL, 0, 0))
                break;

            DispatchMessage(&msg);
        }
		else
		{
			Update();

			SwapBuffers(g_hdc);
		}
    }
    return 0;
}// End WinMain(..)



/***************************************************************************/
/*                                                                         */
/* CreateGl()                                                              */
/* Called once at the start of our program to setup game.                  */
/*                                                                         */
/***************************************************************************/
void CreateGl( void )
{

	PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		24,
		0,0,0,0,0,0,0,0,0,0,0,0,0, // useles parameters
		16,
		0,0,0,0,0,0,0
	};


	GLuint  pixelFormat;

	g_hdc = GetDC(g_hwnd);

	pixelFormat = ChoosePixelFormat(g_hdc, &pfd);
	SetPixelFormat(g_hdc, pixelFormat, &pfd);

	g_hrc = wglCreateContext(g_hdc);

	wglMakeCurrent(g_hdc, g_hrc);

	ShowWindow(g_hwnd, SW_SHOW);
	SetForegroundWindow(g_hwnd);
	SetFocus(g_hwnd);

	glEnable(GL_DEPTH_TEST);


	GLfloat LightAmbient[]=  { 0.6f, 0.6f, 0.6f, 1.0f };	// Ambient Light Values, the values define the amount of P.C light
	GLfloat LightDiffuse[]=	 { 0.8f, 0.8f, 0.8f, 1.0f };	// Diffuse Light Values
	GLfloat LightPosition[]= { 0.0f, 0.0f, 0.0f, 1.0f };	// Light Position, x, y, z and last leave at 1.0 (code junk)


	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);	
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);		
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);	
	
	glEnable(GL_LIGHT1);  
	glEnable (GL_LIGHTING);

}//End CreateGl(..)




/***************************************************************************/
/*                                                                         */
/* ReleaseGl()                                                             */
/* Release any allocated memory, and graphics, resources etc before        */
/* our program finally terminates.                                         */
/*                                                                         */
/***************************************************************************/
void ReleaseGl( void )
{
    wglMakeCurrent(NULL,NULL);

	wglDeleteContext(g_hrc);

	ReleaseDC(g_hwnd, g_hdc);

	DestroyWindow(g_hwnd);

}// End ReleaseGl(..)



/***************************************************************************/
/*                                                                         */
/* DebugPrint()                                                            */
/*  Used to display simple debug information to the screen.  Not for       */
/*  optimised use - due to the fact that it creates/releases resources     */
/*  each time round it is called.                                          */
/*                                                                         */
/***************************************************************************/
void DebugPrint( char * str, float x, float y)
{
	GLuint fontbase; 

	HFONT   font;                              // Windows Font ID 
    HFONT   oldfont;                           // Used For Good House Keeping 

	fontbase = glGenLists(256);					// Storage For 96 Characters 

	font = CreateFont(-13,						// Height Of Font 
						0,						// Width Of Font 
						0,						// Angle Of Escapement 
						0,						// Orientation Angle 
						FW_BOLD,				// Font Weight 
						FALSE,					// Italic 
						FALSE,					// Underline 
						FALSE,					// Strikeout 
						ANSI_CHARSET,			// Character Set Identifier 
						OUT_TT_PRECIS,			// Output Precision 
						CLIP_DEFAULT_PRECIS,		// Clipping Precision 
						ANTIALIASED_QUALITY,		// Output Quality 
						FF_DONTCARE|DEFAULT_PITCH,	// Family And Pitch 
						"Courier New");				// Font Name 

	oldfont = (HFONT)SelectObject(g_hdc, font);		// Selects The Font We Want 
	wglUseFontBitmaps(g_hdc, 0, 256, fontbase);		// Builds 256 Characters 
	SelectObject(g_hdc, oldfont);					// Selects The Font We Want 
	DeleteObject(font);								// Delete The Font 


	glMatrixMode(GL_MODELVIEW);		
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();	
	glOrtho(0, 640, 0, 480, -1, 1);

	glScalef(1.0f, 1.0f, 1.0f);
	//glLoadIdentity();					
	glTranslatef(0.0f,450.0f,-1.0f);


	glColor3f (1.0f, 1.0f, 1.0f);

	glRasterPos2f (x,   // x pos
				   -y); // y pos, negatated so it goes from top to bottom


	glPushAttrib(GL_LIST_BIT);						// Pushes The Display List Bits 
	glListBase(fontbase);							// Sets The Base Character to 0 
	glCallLists((GLsizei)strlen(str), GL_UNSIGNED_BYTE, str);   // Draws The Display List Text 
	glPopAttrib();									// Pops The Display List Bits 


	glDeleteLists(fontbase, 256);					// Delete All 256 Characters 
}//End DebugPrint(..)

