/***************************************************************************/
/*                                                                         */
/*  File: Tank.cpp                                                         */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/
/*

*/
/***************************************************************************/

#include "Tank.h"

#include <windows.h>    // opengl has issues if we don't include it!
#include <GL/gl.h>
//#include <GL/glu.h>	// gluLookAt(..), glPerspective(..)
#include <stdio.h>		// sprintf(..)

#include "Vector3.h"
#include "Land.h"		// Land class
#include "GLDraw.h"
#include "Matrix4.h"
#include "Game.h"
#include "Models.h"

// Simple function to determine the angle of rotation from two vector directions
float AngleXZ(VECTOR3 v0, VECTOR3 v1) // Around Y
{
	v0.y = v1.y = 0.0f;

	float length0 = v0.length();
	float length1 = v1.length();

	// We have something wrong here, a vector can't
	// have a length 0.0f
	if ( (length0==0.0f) || (length1==0.0f) )
	{
		return 0.0f;
	}

	// Work out the arccosine of the dot product
	// of the normalized vecttors
	v0.normalize();
	v1.normalize();

	// Dot product gives:
	// val = |A||B|cos(ang)
	// so if A and B are both length 1, then we have val = cos(ang)

	float val = v0.dot(v1);

	// Check for sneeky float errors
	if (val < -1.0f) val =-1.0f;
	if (val >  1.0f) val = 1.0f;

	float angle = acosf(val);

	// Use the cross product to determine if its left or right
	VECTOR3 cross = v0.cross(v1);

	if (cross.y >= 0.0f)
	{
		angle; // turn left
	}
	else
	{
		angle*=-1; // turn right
	}

	return angle;

}// End AngleXZ(..)


float AngleXY(VECTOR3 v0, VECTOR3 v1) // Around Z
{
	v0.z = v1.z = 0.0f;

	float length0 = v0.length();
	float length1 = v1.length();

	// We have something wrong here, a vector can't
	// have a length 0.0f
	if ( (length0==0.0f) || (length1==0.0f) )
	{
		return 0.0f;
	}

	// Work out the arccosine of the dot product
	// of the normalized vecttors
	v0.normalize();
	v1.normalize();

	// Dot product gives:
	// val = |A||B|cos(ang)
	// so if A and B are both length 1, then we have val = cos(ang)

	float val = v0.dot(v1);

	// Check for sneeky float errors
	if (val < -1.0f) val =-1.0f;
	if (val >  1.0f) val = 1.0f;

	float angle = acosf(val);

	// Use the cross product to determine if its left or right
	VECTOR3 cross = v0.cross(v1);

	if (cross.z >= 0.0f)
	{
		angle*=-1.0f; // turn left
	}
	else
	{
		angle*=1.0f; // turn right
	}

	return angle;
}// End AngleXY(..)


float AngleYZ(VECTOR3 v0, VECTOR3 v1) // Around X
{
	v0.x = v1.x = 0.0f;

	float length0 = v0.length();
	float length1 = v1.length();

	// We have something wrong here, a vector can't
	// have a length 0.0f
	if ( (length0==0.0f) || (length1==0.0f) )
	{
		return 0.0f;
	}

	// Work out the arccosine of the dot product
	// of the normalized vecttors
	v0.normalize();
	v1.normalize();

	// Dot product gives:
	// val = |A||B|cos(ang)
	// so if A and B are both length 1, then we have val = cos(ang)

	float val = v0.dot(v1);

	// Check for sneeky float errors
	if (val < -1.0f) val =-1.0f;
	if (val >  1.0f) val = 1.0f;

	float angle = acosf(val);

	// Use the cross product to determine if its left or right
	VECTOR3 cross = v0.cross(v1);

	if (cross.x >= 0.0f)
	{
		angle*=-1.0f; // turn left
	}
	else
	{
		angle*=1.0f; // turn right
	}

	return angle;
}

/***************************************************************************/

void Tank::Update()
{
	if (m_land==NULL) return;

	m_bullets.Update();

	float centreHeight = m_land->GetHeight(m_pos.x, m_pos.z);
	m_dir.y = 0.0f;
	m_dir.normalize();
	m_pos.y = centreHeight;
	m_centreHeight = centreHeight;

	float angdiff = AngleXZ(m_gradualDir, m_dir);
	float angref = AngleXZ(VECTOR3(0,0,1), m_gradualDir);
	float slerp = angref + g_timeStep*1.2f*(angdiff);


	MATRIX4 rotY;
	rotY.RotateY(slerp);
	m_gradualDir = VECTOR3(0,0,1);
	TransformCoord(&m_gradualDir, &m_gradualDir, &rotY);
	m_gradualDir.y = 0.0f;
	m_gradualDir.normalize();

	if (this!=&g_TankPlayer)
	{
		float dirAng = acosf( m_dir.dot(m_gradualDir) );
		if ( abs(dirAng) > 0.2f )
		{
			return;
		}
	}


	float   vel = m_vel + m_force*g_timeStep;
	VECTOR3 pos = m_pos + m_dir*m_vel*g_timeStep;


	bool bCollision = false;

	// Do some simple collision detection
	for (int i=0; i<sizeof(g_Tanks)/sizeof(Tank*); i++)
	{
		if (g_Tanks[i]==this)
		{
			continue;
		}

		VECTOR3 dir = g_Tanks[i]->GetPos() - pos;
		dir.y = 0.0f;

		if (dir.length()<= 0.001f)
		{
			break;
		}

		float gap = g_Tanks[i]->GetRadius() + m_radius;
		float len = dir.length();
		if (len < gap)
		{
			// We have a collision, deal with it

			VECTOR3 d0 = m_dir;
			d0.y=0;
			d0.normalize();
			dir.normalize();

			VECTOR3 cross = dir.cross(d0);
			float clockwise = 1.0f;
	
			if (cross.y<0.0f)
			{
				clockwise=-1.0f;
			}
	

			VECTOR3 up(0,1.0f,0);
			VECTOR3 hitDir = up.cross(dir);
			hitDir -= clockwise*dir*0.05f;
			
			hitDir = hitDir * clockwise;
			hitDir.normalize();

			m_vel = m_vel + m_force*g_timeStep;
			m_pos = m_pos + hitDir*m_vel*g_timeStep;
			m_hit = i;


			{
				dir = g_Tanks[i]->GetPos() - pos;
				float len = dir.length();

				if (len < (gap*0.5f))
				{
					m_pos = g_Tanks[i]->GetPos() + m_dir*gap*1.005f;
				}
			}
			

			// This is a dirty fix - basically we won't go automatically
			// correction direction for our player vehicle, only do it for
			// aitanks
			if (this!=&g_TankPlayer)
			{
				SetDir(hitDir);
			}

			bCollision |= true;
			break;
		}
	}


	if (bCollision==false)
	{
		m_vel = m_vel + m_force*g_timeStep;
		m_pos = m_pos + m_dir*m_vel*g_timeStep;
		m_hit=-1;
	}

	//static float friction = 0.02f;
	//m_vel = m_vel - g_timeStep*friction;
	m_vel = m_vel*0.98f;

}// End Update(..)

/***************************************************************************/

static void DebugRenderTank(const VECTOR3& pos, float height, float angleX, float angleY, float angleZ)
{
	// Using the model values in model.h, we have a simple model of a tank, its
	// quiet high poly, for what you see, 700 tris....just a simple version - would
	// look nicer with texturing, and maybe do 3 versions for differnt level
	// of details (lods), so its not so much of a performance hit when rendering
	// lots of tanks.

	glPushMatrix();

	glTranslated(pos.x, height, pos.z);

	glRotatef(angleX*(180.0f/3.14f),			1.0f, 0.0f, 0.0f);
	
	glRotatef(angleZ*(180.0f/3.14f),			0.0f, 0.0f, 1.0f);
	
	glRotatef(angleY*(180.0f/3.14f),			0.0f, 1.0f, 0.0f);

	float scale = 0.003f;
	glScalef(scale, scale, scale);

	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glBegin(GL_TRIANGLES);


	int numIndices = Tank0s.numIndices;
	int numVerts   = Tank0s.numVerts;
	float* vert    = Tank0s.v;
	int  * inds    = Tank0s.i;

	for (int i=0; i<numIndices*3; i+=3)
	{
		WORD iA = inds[i+0] * 3;
		WORD iB = inds[i+1] * 3;
		WORD iC = inds[i+2] * 3;

		float tAx = vert[iA + 0];
		float tAz = vert[iA + 1];
		float tAy = vert[iA + 2];

		float tBx = vert[iB + 0];
		float tBz = vert[iB + 1];
		float tBy = vert[iB + 2];

		float tCx = vert[iC + 0];
		float tCz = vert[iC + 1];
		float tCy = vert[iC + 2];

		VECTOR3 v0 = VECTOR3(tAx,tAy,tAz);
		VECTOR3 v1 = VECTOR3(tBx,tBy,tBz);
		VECTOR3 v2 = VECTOR3(tCx,tCy,tCz);

		VECTOR3 vA = v1 - v0;
		VECTOR3 vB = v2 - v0;

		VECTOR3 vN = vA.cross(vB);
		vN.normalize();

		glColor3f(1.0f, 1.0f, 0.0f);
		glNormal3f(vN.x,vN.y,vN.z);
		glVertex3f(tAx, tAy, tAz); 

		glColor3f(1.0f, 1.0f, 0.0f);
		glNormal3f(vN.x,vN.y,vN.z);
		glVertex3f(tBx, tBy, tBz); 

		glColor3f(1.0f, 1.0f, 0.0f);
		glNormal3f(vN.x,vN.y,vN.z);
		glVertex3f(tCx, tCy, tCz); 
	}
	glEnd();

	glPopMatrix(); 	
}

/***************************************************************************/

void Tank::Render()
{
	if (m_land==NULL) return;

	// Using the centre of the tank as reference, we work out the four
	// corners of the tank in the x-z plane to determine the four
	// wheels - which we can use to determine the tanks angle on the ground
	// and maybe draw 4 wheels or tracks later on based on these values

	float len = 0.4f;
	float wid = 0.22f;
	VECTOR3 wheels[4];
	enum
	{
		FLEFT  = 0,
		FRIGHT = 1,
		BLEFT  = 2,
		BRIGHT = 3
	};

	VECTOR3 center = VECTOR3(0,0,0);
	wheels[FLEFT]   = center;
	wheels[FRIGHT]  = center;
	wheels[BLEFT]   = center;
	wheels[BRIGHT]  = center;

	wheels[FLEFT].z  += len;
	wheels[FLEFT].x  -= wid;

	wheels[FRIGHT].z += len;
	wheels[FRIGHT].x += wid;

	wheels[BLEFT].z  -= len;
	wheels[BLEFT].x  -= wid;

	wheels[BRIGHT].z -= len;
	wheels[BRIGHT].x += wid;


	m_angY = AngleXZ( VECTOR3(0,0,1), m_gradualDir );

	for (int i=0; i<4; i++)
	{
		VECTOR3& pos = wheels[i];
		MATRIX4 rot;
		rot.RotateY(m_angY);
		TransformCoord(&pos, &pos, &rot);

		pos += m_pos;

		wheels[i].y = m_land->GetHeight(wheels[i]);
	}


	// Y Rotation is simply the direction where going, but the land where
	// on will determine if where wobbling around the x-z plane, as our
	// ground is uneven
	// Use our wheels to determine our approx rotations
	VECTOR3 p0 = wheels[FLEFT];
	VECTOR3 p1 = wheels[BLEFT];
	VECTOR3 p2 = wheels[BRIGHT];;
	VECTOR3 vA = p1 - p0;
	VECTOR3 vB = p2 - p0;
	VECTOR3 vN = -vA.cross(vB);
	vN.normalize();

	// Debug information - draw the line for straight up (0,1,0), and the
	// actual line for the calculated normal for vN which is from our wheels
	/*
	glLineWidth(2.0);
	glPushMatrix();
	glTranslated(0, 0, 0);
	glRotatef(0, 1.0f, 0.0f, 0.0f);
	glRotatef(0, 0.0f, 1.0f, 0.0f);
	glRotatef(0, 0.0f, 0.0f, 1.0f);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glBegin(GL_LINES);
	{
		glColor3f(0xff, 0, 0xff);
		glVertex3f(m_pos.x, m_pos.y, m_pos.z); 

		VECTOR3 pp = m_pos;
		pp = pp + vN*4.0f;
		glColor3f(0xff, 0, 0xff);
		glVertex3f(pp.x, pp.y, pp.z); 

		glColor3f(0xff, 0, 0);
		glVertex3f(m_pos.x, m_pos.y, m_pos.z); 

		pp = m_pos;
		pp = pp + VECTOR3(0,1,0)*4.0f;
		glColor3f(0xff, 0, 0);
		glVertex3f(pp.x, pp.y, pp.z); 
	}
	glEnd();
	glPopMatrix();
	*/




	// Simple test code to check our land height collision detection
	// code is working..
	for (int i=0; i<4; i++)
	{
		VECTOR3& pos = wheels[i];

		glPushMatrix();
		glTranslated(pos.x,pos.y,pos.z);
		float sz = 0.1f;
		glScalef (sz, sz, sz);
		glLineWidth(3.0);
		glColor3d(1.0,1.0,1.0);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // Draw Both front and back facing polygons as lines
		DrawUnitCube();
		glPopMatrix();
	}

	DrawCircleXZ(m_pos.x, m_centreHeight + 0.9f, m_pos.z, m_radius);

	RenderGun();

	m_bullets.Render();


	// Only draw a solid tank for the player if its enabled, till
	// it can be improved so its not a performance hit - to higher poly
	// and needs to have its normals and sizes precalucated once before
	// render
	extern bool g_bSolidTank;
	if (g_bSolidTank)
	{
		// Debug - use this line if we only want to draw the tank for only
		// the player.

		//if (&g_TankPlayer == this)
		{
			m_angX = AngleYZ(vN, VECTOR3(0,1,0));
			m_angZ = AngleXY(vN, VECTOR3(0,1,0));
			
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			DebugRenderTank(m_pos, m_centreHeight, m_angX, m_angY, m_angZ);

			/*
			extern DrawString* g_str;
			if (g_str)
			{
				char buf[300];

				sprintf(buf, "angX: %.2f, (%.2f)", angX, angX*(180/3.14f));
				g_str->Print(buf, 5, 80);

				sprintf(buf, "angZ: %.2f, (%.2f)", angZ, angZ*(180/3.14f));
				g_str->Print(buf, 5, 95);
			}
			*/

		}
	}


}// Render(..)


void Tank::RenderGun()
{
	// length of the gun barrel
	const float lenGunBarrel = 0.7f;
	// the radius of the gun barrel
	const float dim = 0.05f;

	// We will draw the gun barrel as a long extruded rectangle :)

	// Push the current matrix onto the stack so it wont' effect
	// everything else we draw
	glPushMatrix();

	// Move it into world space
	glTranslated(m_pos.x,m_centreHeight, m_pos.z);

	// Using our tanks direction, and its angle from the ground,
	// we tweek the gun
	glRotatef(m_angX * (180.0f/3.14f), 1.0f, 0,		0);
	glRotatef(m_angZ * (180.0f/3.14f), 0,	 0,		1.0f);
	glRotatef(m_angY * (180.0f/3.14f), 0,	 1.0f,	0);
	
	// Modify the gun's direction so it faces up or down in the correct direction
	glRotatef(m_gunUpAngle * (180.0f/3.14f), 1.0f, 0, 0);
	// Size our gun to how big we want it, width, height and depth
	glScalef (dim, lenGunBarrel, dim);
	// Position the barrel, so its base is at 0 (in y)
	glTranslated(0, 1.0f, 0);
	

	//glLineWidth(1.0);
	glColor3d(0.7,0.7,1.0);
	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	DrawUnitCube();
	glPopMatrix();


	// Work out the direction that our bullet will fire

	// We have to work out the angle and direction our bullet will
	// fire, depending on the tanks direction, the angle the barrel
	// is facing up
	VECTOR3 refDir = VECTOR3(0,1,0); // Default dir without any rotation

	// Up-Down Barrel Guns direction
	MATRIX4 rotX;
	rotX.RotateX(m_gunUpAngle);

	// Tanks angles, varies acording to the direction where moving, and
	// the terrain where sitting on
	MATRIX4 rotZZ;
	MATRIX4 rotXX;
	MATRIX4 rotYY;
	rotZZ.RotateZ(m_angZ);
	rotXX.RotateX(m_angX);
	rotYY.RotateY(m_angY);

	MATRIX4 mat;
	// Do the guns barrel up-down angle
	MATRIX4::MatrixMultiply(mat, mat, rotX);

	// Add the tanks world xyz rotation
	MATRIX4::MatrixMultiply(mat, mat, rotYY);
	MATRIX4::MatrixMultiply(mat, mat, rotXX);
	MATRIX4::MatrixMultiply(mat, mat, rotZZ);
	
	
	TransformCoord(&refDir, &refDir, &mat);
	m_gunFireDir = refDir;

#if 0 // Debug Draw the Firing Direction
	glLineWidth(2.0);
	glPushMatrix();
	glTranslated(0, 0, 0);
	glRotatef(0, 1.0f, 0.0f, 0.0f);
	glRotatef(0, 0.0f, 1.0f, 0.0f);
	glRotatef(0, 0.0f, 0.0f, 1.0f);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glBegin(GL_LINES);
	{
		VECTOR3 from = VECTOR3(m_pos.x, m_centreHeight, m_pos.z);
		VECTOR3 to   = from + m_gunFireDir*3.0f;

		glColor3f(0xff, 0, 0xff);
		glVertex3f(from.x, from.y, from.z); 

		glColor3f(0xff, 0, 0xff);
		glVertex3f(to.x, to.y, to.z); 
	}
	glEnd();
	glPopMatrix();
#endif

}//End RenderGun(..)