/***************************************************************************/
/*                                                                         */
/*  File: Tank.h                                                           */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
	Since where mostly dealing with tanks, thought it would be a good idea
	to have a class which is a tank class!
*/
/***************************************************************************/
#ifndef TANK_H
#define TANK_H

#include <windows.h>    // opengl has issues if we don't include it!
//#include <GL/gl.h>
//#include <GL/glu.h>	// gluLookAt(..), glPerspective(..)

#include "Vector3.h"
#include "Land.h"		// Land class
#include "Bullets.h"


float AngleXZ(VECTOR3 v0, VECTOR3 v1);

class Tank
{
protected:

	VECTOR3	m_pos;
	VECTOR3 m_dir;
	VECTOR3 m_gradualDir;
	float   m_vel;
	float	m_force;
	Land*	m_land;
	float	m_radius;
	int		m_hit;
	float	m_centreHeight;
	VECTOR3 m_gunFireDir;
	float   m_gunUpAngle;
	Bullets m_bullets;
	char	m_name[32];
	float	m_angX;
	float	m_angZ;
	float   m_angY;

public:
	Tank()
	{
		m_land=NULL;
		memset(m_name, '\0', sizeof(m_name)-1);
	}

	
	virtual bool Create(Land* land, char* name = NULL)
	{
		m_pos = VECTOR3(0,0,0);
		m_dir = VECTOR3(0,0,1);
		m_gradualDir = VECTOR3(0,0,1);
		m_force = 0.5f;
		m_vel   = 0.0f;
		m_land = land;
		m_radius = 1.0f;
		m_hit = -1;
		m_centreHeight = 0.0f;
		m_gunFireDir = VECTOR3(0,0,1);
		m_gunUpAngle = 3.14f/4.0f;
		if (name)
		{
			strcpy(m_name, name);
		}
		m_angX = 0.0f;
		m_angZ = 0.0f;
		m_angY = 0.0f;
		Update();
		return true;
	}

	virtual void Release()
	{
	}


	virtual void Update();
	virtual void Render();

	void Fire(float strength)
	{
		VECTOR3 from = VECTOR3(m_pos.x, m_centreHeight, m_pos.z);
		from += m_gunFireDir*1.2f;
		m_bullets.AddBullet(from, m_gunFireDir*strength);
	}

	__forceinline VECTOR3 GetPos()   const	
	{
		VECTOR3 tmp = m_pos;
		tmp.y = m_centreHeight;
		return tmp;
	}

	__forceinline VECTOR3 GetGradualDir()	const {return m_gradualDir;}
	__forceinline VECTOR3 GetDir()			const {return m_dir;}
	__forceinline float   GetForce()		const {return m_force;}
	__forceinline float	  GetRadius()		const {return m_radius;}
	__forceinline float   GetGunUpAngle()	const {return m_gunUpAngle;}
	__forceinline VECTOR3 GetGunDir()		const {return m_gunFireDir;}

	__forceinline void SetPos(const VECTOR3& pos)		{m_pos   = pos;}
	__forceinline void SetDir(const VECTOR3& dir)		{m_dir	 = dir;}
	__forceinline void SetGradualDir(const VECTOR3& dir){m_gradualDir = m_dir;}
	__forceinline void SetForce(const float force)		{m_force = force;}
	__forceinline void SetRadius(const float r)			{m_radius = r;}
	__forceinline void SetGunUpAngle(const float ang)	{m_gunUpAngle=ang;}


protected:
	virtual void RenderGun();
};


#endif //TANK_H

