-- --------------------------------------------------------------------------------------
--
-- Raw Data 3d maxscript file exporter
-- Ben Kenwright
-- Email: bkenwright@xbdev.net
--
-- --------------------------------------------------------------------------------------

fn writemesh f obj=
(
	writestring f "// *********\n"
	writestring f "// *\n"
    ss = stringstream ""
	format "// *  stModelData %s\n" obj.name to:ss
	writestring f (ss as string)
	writestring f "// *\n"
	writestring f "// **********\n\n\n"
	
	-- Number of Indices
	numIndices = obj.numfaces

	-- Number of Vertices
	numVerts = obj.numverts
	

	ss = stringstream ""
	name = (obj.name as string)

	-- Write the Indices to File
	format "int %I[] = \n{\n" obj.name to:ss
	writestring f (ss as string)
	
	for i=1 to obj.numfaces do
	(	
		i1=((getFace obj i).z)-1
		i2=((getFace obj i).y)-1
		i3=((getFace obj i).x)-1
			
		ss = stringstream ""
		format "\t %, %, %,\n" (i1 as integer) (i2 as integer) (i3 as integer) to:ss	
		writestring f (ss as string)
	)
	writestring f "\n};\n\n"
	
	-- Write the Vertices to File
	ss = stringstream ""
	format "float %V[] = \n{\n" obj.name to:ss
	writestring f (ss as string)
	for i=1 to obj.numverts do
	(
		local v=in coordsys world(getvert obj i)
		
		ss = stringstream ""
		format "\t %, %, %,\n" (v.x as string) (v.y as string) (v.z as string) to:ss	
		writestring f (ss as string)
	)
	writestring f "\n};\n\n"
	
	-- Write out the single structure define for the 3d object
	ss = stringstream ""
	format "stModelData %s = \n{\n" obj.name to:ss
	writestring f (ss as string)
	
	ss = stringstream ""
	format "\t %, // Number Indices\n" (numIndices as string) to:ss	
	writestring f (ss as string)
	
	ss = stringstream ""
	format "\t %, // Number Vertices\n" (numVerts as string) to:ss	
	writestring f (ss as string)
	

	ss = stringstream ""
	format "\t%I,\n" obj.name to:ss
	writestring f (ss as string)
	
	ss = stringstream ""
	format "\t%V\n" obj.name to:ss
	writestring f (ss as string)

	
	writestring f "\n};\n"
)

-- --------------------------------------------------------------------------------------

fn writefile obj filename=
(
	if obj==undefined do return"No object"
	if (classof obj)!=editable_mesh do return "Object is not a mesh"
	
	f=fopen filename "wt"
	
	writestring f "\n\n"
	writestring f "// Raw Data .h File Exporter \n// www.xbdev.net\n\n"

	writestring f "struct stModelData\n"
	writestring f "{\n"
	writestring f "   int   numIndices;\n"
	writestring f "   int   numVerts;\n"
	writestring f "   int   *i;\n";
	writestring f "   float *v;\n";
	writestring f "};\n\n"
	
	writemesh f obj
	
	fclose f
)

-- --------------------------------------------------------------------------------------

fn addcheck cb=
(
 	if classof cb==string do messagebox cb title:"3D Data Exporter Error"
)

-- --------------------------------------------------------------------------------------

utility XExport "3D Data Txt Mesh Exporter"
(
	group "Export"
	(
		button bexport "     Export    "
	)
	group "About"
	(
		label titleLabel	"Raw Data Txt Exporter v0.01"
		HyperLink addy "by Ben Kenwright" align:#center address:"mailto:bkenwright@xbdev.net" color:(color 0 100 0) hoverColor:(color 0 0 100)
		HyperLink me "HomePage" align:#center address:"www.xbdev.net" color:(color 0 100 0) hoverColor:(color 0 0 100)

	)
	
	on bexport pressed do
	( 
		newfilename=getsavefilename caption:"Save .x" \
			newfilename:lastfile \
			types:"HeaderFile .h (*.h)|*.h|All Files (*.*)|*.*|"
		if newfilename!=undefined do 
		(				
			addcheck (writefile $ newfilename)
			lastfile=newfilename
		)
	)
)

